import logging
import os
import pdb
import math

import palettable
import numpy as np
import matplotlib.pyplot as plt

import constants
import pygeoutil.util as util
import plot


def process_MIAMI_LU():
    """
    Convert MIAMI LU AGB and NPP estimates to quarter degree
    :return:
    """
    # Read in MIAMI-LU data
    mlu_data = util.open_or_die(constants.miami_lu_nc)

    # Get last year for which biomass is extracted
    lyr = mlu_data.variables['biomass'].shape[0] - 1

    # Extract agb and npp layers and convert to quarter degree format
    mlu_vba = mlu_data.variables['biomass'][lyr, :, :]
    mlu_vba = mlu_vba.repeat(2, 0).repeat(2, 1)
    mlu_npp = mlu_data.variables['aa_NPP'][lyr, :, :]
    mlu_npp = mlu_npp.repeat(2, 0).repeat(2, 1)

    # Output
    np.savetxt(constants.out_dir + 'miami_vba_quarter_deg.txt', mlu_vba, fmt='%1.1f')
    np.savetxt(constants.out_dir + 'miami_npp_quarter_deg.txt', mlu_npp, fmt='%1.1f')

    # Read in previous estimates
    prev_mlu_npp = util.open_or_die(constants.miami_npp)
    prev_mlu_vba = util.open_or_die(constants.miami_vba)

    # Compare
    diff_vba = mlu_vba - prev_mlu_vba
    diff_npp = mlu_npp - prev_mlu_npp

    # Plot
    xaxis_min, xaxis_max, xaxis_step = util.get_ascii_plot_parameters(mlu_vba)
    plot.plot_ascii_map(mlu_vba, constants.out_dir, xaxis_min, xaxis_max, xaxis_step, plot_type='sequential',
                        xlabel=r'$Biomass\ (kg\ C/m^{2})$', title='', var_name='Biomass', skiprows=0)

    xaxis_min, xaxis_max, xaxis_step = util.get_ascii_plot_parameters(prev_mlu_vba)
    plot.plot_ascii_map(prev_mlu_vba, constants.out_dir, 0.0, 27.0, 3.0, plot_type='sequential',
                        xlabel=r'$Biomass\ (kg\ C/m^{2})$', title='', var_name='prev_Biomass', skiprows=0)

    xaxis_min, xaxis_max, xaxis_step = util.get_ascii_plot_parameters(mlu_npp)
    plot.plot_ascii_map(mlu_npp, constants.out_dir, xaxis_min, xaxis_max, xaxis_step, plot_type='sequential',
                        xlabel=r'$NPP\ (kg\ C/m^{2})$', title='', var_name='NPP', skiprows=0)

    # Difference
    xaxis_min, xaxis_max, xaxis_step = util.get_ascii_plot_parameters(diff_vba)
    plot.plot_ascii_map(diff_vba, constants.out_dir, xaxis_min, xaxis_max, xaxis_step, plot_type='diverging',
                        xlabel=r'$(New - Old)\ Biomass\ (kg\ C/m^{2})$', title='', var_name='Difference_Biomass', skiprows=0)

    xaxis_min, xaxis_max, xaxis_step = util.get_ascii_plot_parameters(diff_npp)
    plot.plot_ascii_map(diff_npp, constants.out_dir, xaxis_min, xaxis_max, xaxis_step, plot_type='diverging',
                        xlabel=r'$(New - Old)\ NPP\ (kg\ C/m^{2})$', title='', var_name='Difference_NPP', skiprows=0)


def plot_miami_3D(max_temp=40.5, max_prcp=3000.):
    """

    Args:
        max_temp:
        max_prcp:

    Returns:

    """
    step = 0.5

    temp = np.arange(0.0, max_temp, step)
    prcp = np.arange(0.0, max_prcp, (max_prcp - 0)/(max_temp/step))
    x_surf, y_surf = np.meshgrid(temp, prcp / 10.)

    all = np.transpose([np.tile(temp, len(prcp)), np.repeat(prcp, len(temp))])

    temp = all[:, 0]
    prcp = all[:, 1]
    nppt = 3000. / (1 + np.exp(1.315 - 0.119 * temp))
    nppp = 3000. * (1 - np.exp(-0.000664 * prcp))

    npp = np.minimum(nppt, nppp) * 0.45/1000.

    fig = plt.figure()
    ax = fig.gca(projection='3d')

    #ax.scatter(temp, prcp/10., npp, alpha=0.5, color=None)
    #ax.plot(temp, np.repeat(max(prcp / 10.), len(temp)), nppt * 0.45/1000., color='black')
    #ax.plot(np.repeat(max(temp), len(npp)), prcp/10., nppp * 0.45/1000., color='black')

    surf = ax.plot_surface(x_surf, y_surf, np.reshape(npp, (max_temp/step, max_temp/step)), linewidth=0.2,
                           cmap=palettable.colorbrewer.sequential.Greens_9.mpl_colormap)

    ax.set_xlabel('MAT ($^\circ$C)', fontsize=10)
    ax.set_ylabel('MAP ($cm\ yr^{-1}$)', fontsize=10)
    ax.set_zlabel('NPP $(kg\ C\ m^{-2}\ yr^{-1})$', fontsize=10)

    ax.set_xlim3d(0, 40.)
    ax.set_ylim3d(0, max(prcp/10.))
    ax.set_zlim3d(0, 3. * 0.45)

    for label in ax.yaxis.get_ticklabels()[::2]:
        label.set_visible(False)

    cbar = fig.colorbar(surf, shrink=0.5, aspect=10, orientation='horizontal')
    cbar.ax.set_xlabel('NPP $(kg\ C\ m^{-2}\ yr^{-1})$')

    for label in cbar.ax.xaxis.get_ticklabels()[::2]:
        label.set_visible(False)

    plt.show()


if __name__ == '__main__':
    process_MIAMI_LU()
