import logging
import os
import pdb

from enum import Enum
from tqdm import tqdm
import numpy as np
import pandas as pd

import constants
import pygeoutil.util as util


RES_LUH1 = 0.5

class LU(Enum):
    """
    Land-use types (main).
    """
    __order__ = 'gothr gsecd gurbn gcrop gpast gssma gssmb gvbh1 gvbh2 gsbh1 gsbh2 gsbh3 all'
    gothr, gsecd, gurbn, gcrop, gpast, gssma, gssmb, gvbh1, gvbh2, gsbh1, gsbh2, gsbh3, all = range(13)


def add_to_list(arr, start_yr, end_yr):
    """

    Args:
        arr:
        start_yr:
        end_yr:

    Returns:

    """
    # TODO: Make more robust, years hard coded in right now
    new_arr = []
    for yr in range(850, start_yr):
        new_arr.extend([np.nan])

    new_arr.extend(arr)

    for yr in range(end_yr, 2015):
        new_arr.extend([np.nan])

    return new_arr


def get_lu_array(lu_type, year, name_LUH1='LUHa_u2.v1', subset_arr=None):
    """
    Get array of LU category
    :param lu_type:
    :param year:
    :param subset_arr:
    :return:
    """
    try:
        arr = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/' + lu_type.name + '.'
                               + str(int(year)) + '.txt', skiprows=6)

        arr_subset = util.extract_from_ascii(arr, res=RES_LUH1, subset_arr=subset_arr)
    except:
        arr_subset = np.nan

    return arr_subset


def get_lu_area(lu_type, year, subset_arr=None):
    """

    Args:
        lu_type:
        year:
        subset_arr:

    Returns:

    """
    carea = util.open_or_die(constants.CELL_AREA_H)

    return np.ma.sum(get_lu_array(lu_type, year, subset_arr=subset_arr) * carea)


def get_lu_area_ts(lu_type, years, name_LUH1='LUHa_u2.v1', subset_arr=None):
    """
    Get land use area for a land use state for user specified years
    Args:
        lu_type:
        years:
        subset_arr:

    Returns:
        a list of areas
    """
    carea = util.open_or_die(constants.CELL_AREA_H)
    areas_lu = []

    for year in years:
        areas_lu.append(np.ma.sum(get_lu_array(lu_type, year, name_LUH1=name_LUH1, subset_arr=subset_arr) * carea))

    return areas_lu


def return_transition_name(name_state):
    """
    E.g. return 'c' for state 'crop'
    Args:
        name_state:

    Returns:

    """
    if name_state == 'crop':
        return 'c'
    elif name_state == 'pasture':
        return 'p'
    elif name_state == 'urban':
        return 'u'
    elif name_state == 'secondary':
        return 's'
    elif name_state == 'primary':
        return 'v'
    else:
        print('State: ' + name_state + ' does not exist')


def get_name_state_from_LUH1_enum(name_LUH1_state):
    """
    Return english name based on LUH1 enum name
    Args:
        name_LUH1_state:

    Returns:

    """
    if name_LUH1_state == LU.gcrop:
        return 'crop'
    elif name_LUH1_state == LU.gpast:
        return 'pasture'
    elif name_LUH1_state == LU.gurbn:
        return 'urban'
    elif name_LUH1_state == LU.gsecd:
        return 'secondary'
    elif name_LUH1_state == LU.gothr:
        return 'primary'
    else:
        print('State: ' + name_LUH1_state + ' does not exist')


def get_transition(source, target, year, name_LUH1='LUHa_u2.v1', subset_arr=None):
    """
    Add up all transitions between source and target for given year
    Args:
        source:
        target:
        year:
        subset_arr:

    Returns:

    """
    src_lus = return_transition_name(source)
    tgt_lus = return_transition_name(target)

    # Return np.nan if either source == target or target == primary
    if source == target or target == 'primary':
        return np.nan

    arr = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gfl' + src_lus +
                           tgt_lus + '.' + str(year) + '.txt', skiprows=6)

    arr_trans = util.extract_from_ascii(arr, res=RES_LUH1, subset_arr=subset_arr)

    return arr_trans


def get_transitions_into_state(source, year, name_LUH1='LUHa_u2.v1', subset_arr=None):
    """
    Sum up all transitions INTO source
    Args:
        source:
        year:
        name_LUH1:
        subset_arr:

    Returns:

    """
    list_targets = ['crop', 'pasture', 'urban', 'secondary', 'primary']
    arr_trans = np.zeros_like(constants.CELL_AREA_H)

    for tgt in list_targets:
        arr_trans = arr_trans + get_transition(tgt, source, year, name_LUH1, subset_arr)

    return arr_trans


def get_transitions_from_state(source, year, name_LUH1='LUHa_u2.v1', subset_arr=None):
    """
    Sum up all transitions OUT of source
    Args:
        source:
        year:
        name_LUH1:
        subset_arr:

    Returns:

    """
    list_targets = ['crop', 'pasture', 'urban', 'secondary']
    arr_trans = np.zeros_like(constants.CELL_AREA_H)

    for tgt in list_targets:
        arr_trans = arr_trans + get_transition(source, tgt, year, name_LUH1, subset_arr)

    return arr_trans


def get_glm_forest_area(year, subset_arr=None):
    """
    Return forest area (secd + othr) for given year
    Args:
        year:
        subset_arr:

    Returns:

    """
    return get_lu_area(LU.gothr, year, subset_arr=subset_arr) + get_lu_area(LU.gsecd, year, subset_arr=subset_arr)


def wood_clearing_for_ag(years, subset_arr=None):
    """
     For each grid-cell this is the sum of transitions from primary to cropland and pasture, multiplied by the cell
     area and the potential biomass density PLUS the sum of transitions from secondary to cropland and pasture,
     multiplied by the cell area and secondary mean biomass density. To get a global total, sum over all grid-cells.
    Args:
        years:
        subset_arr:

    Returns:

    """
    carea = util.open_or_die(constants.CELL_AREA_H)

    halfdeg = util.open_or_die(constants.input_dir + os.sep +
                               '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

    vba = halfdeg * 0.75  # Get above ground biomass
    vba = vba * (vba > 0.01) + (vba < 0.01) * 0.01  # Set least value of vba to 0.01

    fnf = np.copy(vba)
    fnf[fnf < 2.0] = 0.0  # Biomass defn of forest: > 2.0 kg C/m^2
    fnf[fnf > 0.0] = 1.0

    sum_wc_ag = 0.0
    wc_ag = []
    for yr in tqdm(years, desc='LUH1_wood_clearing_for_ag', disable=(len(years) < 2)):
        # 2d array showing area of land transitioning from primary land to pasture and cropland
        primary_to_pasture = get_transition('primary', 'pasture', year=yr, subset_arr=subset_arr)
        primary_to_cropland = get_transition('primary', 'crop', year=yr, subset_arr=subset_arr)

        # 2d array showing area of land transitioning from secondary land to pasture and cropland
        scnd_to_pasture = get_transition('secondary', 'pasture', year=yr, subset_arr=subset_arr)
        scnd_to_cropland = get_transition('secondary', 'crop', year=yr, subset_arr=subset_arr)

        val = np.ma.sum(((primary_to_pasture + primary_to_cropland) * vba +
                         (scnd_to_pasture + scnd_to_cropland) * get_lu_array(LU.gssmb, year=yr,
                                                                             subset_arr=subset_arr)) * fnf * carea)

        sum_wc_ag += val
        wc_ag.extend([val])

    return sum_wc_ag, wc_ag


def compute_wh(years, subset_arr=None):
    """
    Compute wood harvesting (sum of wood harvest biomass (Pg C) and time-series of wood-harvest biomass)
    Args:
        years:
        subset_arr:

    Returns:
        sum of wood harvest biomass (Pg C) and time-series of wood-harvest biomass

    """
    ts_wh = []
    for yr in years:
        # wood harvest biomass from primary forest
        primf_bioh = get_lu_array(LU.gvbh1, year=yr, subset_arr=subset_arr)

        # wood harvest biomass from primary non-forest
        primn_bioh = get_lu_array(LU.gvbh2, year=yr, subset_arr=subset_arr)

        # wood harvest biomass from secondary mature forest
        secmf_bioh = get_lu_array(LU.gsbh1, year=yr, subset_arr=subset_arr)

        # wood harvest biomass from secondary young forest
        secyf_bioh = get_lu_array(LU.gsbh2, year=yr, subset_arr=subset_arr)

        # wood harvest biomass from secondary non-forest
        secnf_bioh = get_lu_array(LU.gsbh3, year=yr, subset_arr=subset_arr)

        val = np.ma.sum(primf_bioh + primn_bioh + secmf_bioh + secnf_bioh + secyf_bioh) * constants.KG_TO_PG

        ts_wh.extend([val])

    return ts_wh


def human_impact_land(years, subset_arr=None):
    """

    :param years:
    :param subset_arr:
    :return:
    """

    area_global = 0.0
    area_impacted_human = 0.0

    for yr in years:
        # Get the ascii array
        othr = get_lu_array(LU.gothr, year=yr, subset_arr=subset_arr)
        secd = get_lu_array(LU.gsecd, year=yr, subset_arr=subset_arr)
        urbn = get_lu_array(LU.gurbn, year=yr, subset_arr=subset_arr)
        past = get_lu_array(LU.gpast, year=yr, subset_arr=subset_arr)
        crop = get_lu_array(LU.gcrop, year=yr, subset_arr=subset_arr)

        area_global += np.ma.sum(othr + secd + urbn + past + crop)

        area_impacted_human += np.ma.sum(crop + past + secd + urbn)

    return (area_impacted_human * 100.0) / area_global


def get_glm_scnd_area(yr, subset_arr=None):
    """

    Args:
        yr:
        subset_arr:

    Returns:

    """
    carea = util.open_or_die(constants.CELL_AREA_H)

    halfdeg = util.open_or_die(constants.input_dir + os.sep +
                               '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

    vba = halfdeg * 0.75  # Get above ground biomass
    vba = vba * (vba > 0.01) + (vba < 0.01) * 0.01  # Set least value of vba to 0.01

    fnf = np.copy(vba)
    fnf[fnf < 2.0] = 0.0  # Biomass defn of forest: > 2.0 kg C/m^2
    fnf[fnf > 0.0] = 1.0

    # Get the ascii array
    secd = get_lu_array(LU.gsecd, year=yr, subset_arr=subset_arr)

    scnd_area = np.ma.sum(secd * carea)
    scnd_frst_area = np.ma.sum(secd * fnf * carea)
    scnd_non_frst_area = scnd_area - scnd_frst_area

    return scnd_area, scnd_frst_area, scnd_non_frst_area


def get_glm_LUH1_forest_area(yr, subset_arr=None):
    """
    Return forest area (secdf + primf) for given year
    :param yr:
    :param subset_arr:
    :return:
    """
    carea = util.open_or_die(constants.CELL_AREA_H)

    # Get the ascii array
    othr = get_lu_array(LU.gothr, year=yr, subset_arr=subset_arr)
    secd = get_lu_array(LU.gsecd, year=yr, subset_arr=subset_arr)

    # forest area
    return np.ma.sum((secd + othr) * carea)


def ag_land_in_sc_LUH1(years, subset_arr=None):
    """

    :param years:
    :param subset_arr:
    :return:
    """
    area_ag_to_sc = 0.0

    # Butler map is constant throughout time
    carea = util.open_or_die(constants.CELL_AREA_H)

    # Get ascii file corresponding to Butler map and convert to correct resolution i.e from quarter deg to half deg
    arr_sc = util.open_or_die(constants.ASC_HALF_DEG_BUTLER)

    for yr in years:
        # Get ascii file corresponding to crop and subset if needed
        past = get_lu_array(LU.gpast, year=yr, subset_arr=subset_arr)
        crop = get_lu_array(LU.gcrop, year=yr, subset_arr=subset_arr)

        area_ag_to_sc += np.ma.sum(arr_sc * (crop + past) * carea)

    return area_ag_to_sc/15.0


def get_MLU_LUH1_agb(subset_arr=None, do_forest=True):
    """
    Returns potential AGB on FOREST areas
    Args:
        subset_arr:
        do_forest:

    Returns:

    """
    carea = util.open_or_die(constants.CELL_AREA_H)

    halfdeg = util.open_or_die(constants.input_dir + os.sep +
                               '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

    vba = halfdeg * 0.75  # Get above ground biomass
    vba = vba * (vba > 0.01) + (vba < 0.01) * 0.01  # Set least value of vba to 0.01

    if do_forest:
        fnf = np.copy(vba)
        fnf[fnf < 2.0] = 0.0  # Biomass defn of forest: > 2.0 kg C/m^2
        fnf[fnf > 0.0] = 1.0
    else:
        fnf = 1.0

    if subset_arr is not None:
        return np.ma.sum(vba * carea * fnf * subset_arr)
    else:
        return np.ma.sum(vba * carea * fnf)


def get_LUH1_biomass(years, only_forest=True, ulat=90.0, llat=-90.0, llon=-180.0, rlon=180.0, name_LUH1='LUHa_u2.v1',
                     subset_arr=None):
    """

    Args:
        years:
        only_forest:
        ulat:
        llat:
        llon:
        rlon:
        subset_arr:

    Returns:

    """
    carea = util.open_or_die(constants.CELL_AREA_H)
    if subset_arr:
        carea = carea * subset_arr
    
    halfdeg = util.open_or_die(constants.input_dir + os.sep +
                               '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

    vba = halfdeg * 0.75  # Get above ground biomass
    vba = vba*(vba > 0.01) + (vba < 0.01)*0.01  # Set least value of vba to 0.01

    fnf = np.copy(vba)
    fnf[fnf < 2.0] = 0.0  # Biomass defn of forest: > 2.0 kg C/m^2
    fnf[fnf > 0.0] = 1.0

    sum_biom = 0.0
    # Spatially subset carea, fnf and vba
    carea = util.extract_from_ascii(carea, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                    subset_arr=subset_arr)
    vba = util.extract_from_ascii(vba, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1, subset_arr=subset_arr)
    fnf = util.extract_from_ascii(fnf, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1, subset_arr=subset_arr)

    for yr in years:
        # Get the ascii array
        arr_ssmb = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gssmb.' +
                                    str(yr) + '.txt', skiprows=6)
        arr_othr = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gothr.' +
                                    str(yr) + '.txt', skiprows=6)
        arr_secd = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gsecd.' +
                                    str(yr) + '.txt', skiprows=6)
        # Subset by lat lon boundary
        ssmb = util.extract_from_ascii(arr_ssmb, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                       subset_arr=subset_arr)
        othr = util.extract_from_ascii(arr_othr, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                       subset_arr=subset_arr)
        secd = util.extract_from_ascii(arr_secd, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                       subset_arr=subset_arr)

        # Secondary biomass = secondary * cell_area * secondary biomass density
        if not only_forest:
            secdf_biom = np.ma.sum(secd * carea * ssmb)
        else:
            secdf_biom = np.ma.sum(secd * carea * fnf * ssmb)

        # Primary biomass = primary * cell_area * primary biomass density
        if not only_forest:
            primf_biom = np.ma.sum(othr * carea * vba)
        else:
            primf_biom = np.ma.sum(othr * carea * fnf * vba)

        sum_biom += primf_biom + secdf_biom

    return sum_biom


def get_cumulative_emissions(start_yr, end_yr, ulat=90.0, llat=-90.0, llon=-180.0, rlon=180.0, name_LUH1='LUHa_u2.v1',
                             subset_arr=None):
    """
    Cumulative emissions is computed as difference between potential or near potential biomass in start_yr and
    biomass in current year (end_yr)
    It is a NET quantity, since we only take into account emissions and NOT regrowth
    Args:
        start_yr:
        end_yr:
        ulat:
        llat:
        llon:
        rlon:
        subset_arr:

    Returns:

    """
    list_tot_biom = []

    carea = util.open_or_die(constants.CELL_AREA_H)
    if subset_arr:
        carea = carea * subset_arr

    halfdeg = util.open_or_die(constants.input_dir + os.sep +
                               '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

    vba = halfdeg * 0.75  # Get above ground biomass
    vba = vba * (vba > 0.01) + (vba < 0.01) * 0.01  # Set least value of vba to 0.01

    fnf = np.copy(vba)
    fnf[fnf < 2.0] = 0.0  # Biomass defn of forest: > 2.0 kg C/m^2
    fnf[fnf > 0.0] = 1.0

    # Spatially subset carea, fnf and vba
    carea = util.extract_from_ascii(carea, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                    subset_arr=subset_arr)
    vba = util.extract_from_ascii(vba, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1, subset_arr=subset_arr)
    fnf = util.extract_from_ascii(fnf, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1, subset_arr=subset_arr)

    for idx, yr in enumerate([start_yr, end_yr]):
        # Get the ascii array
        arr_ssmb = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gssmb.' +
                                    str(yr) + '.txt', skiprows=6)
        arr_othr = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gothr.' +
                                    str(yr) + '.txt', skiprows=6)
        arr_secd = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gsecd.' +
                                    str(yr) + '.txt', skiprows=6)

        # Subset by lat lon boundary
        ssmb = util.extract_from_ascii(arr_ssmb, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                       subset_arr=subset_arr)
        othr = util.extract_from_ascii(arr_othr, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                       subset_arr=subset_arr)
        secd = util.extract_from_ascii(arr_secd, ulat=ulat, llat=llat, llon=llon, rlon=rlon, res=RES_LUH1,
                                       subset_arr=subset_arr)

        # Secondary biomass = secondary * cell_area * secondary biomass density
        secdf_biom = np.ma.sum(secd * carea * ssmb)

        # Primary biomass = primary * cell_area * primary biomass density
        primf_biom = np.ma.sum(othr * carea * vba)

        # Add secondary and primary biomass
        list_tot_biom.append(np.ma.sum(secdf_biom + primf_biom))

    # Compute difference in biomass
    cum_emissions = list_tot_biom[1] - list_tot_biom[0]

    return cum_emissions


def get_LUH1_secma(do_forest=True, yr=2005, name_LUH1='LUHa_u2.v1', subset_arr=None):
    """

    Args:
        do_forest:
        yr:
        subset_arr:

    Returns:

    """
    ssma = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gssma.' + str(yr) +
                            '.txt', skiprows=6)

    # Subset by forest/non-forest map
    if do_forest:
        halfdeg = util.open_or_die(constants.input_dir + os.sep +
                                   '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

        vba = halfdeg * 0.75  # Get above ground biomass
        vba = vba * (vba > 0.01) + (vba < 0.01) * 0.01  # Set least value of vba to 0.01

        fnf = np.copy(vba)
        fnf[fnf < 2.0] = 0.0  # Biomass defn of forest: > 2.0 kg C/m^2
        fnf[fnf > 0.0] = 1.0

        ssma = np.ma.masked_where(fnf <= 0.0, ssma)

    return ssma


def diag_LUH1(start_yr=1500, end_yr=2005, do_extend=True, name_LUH1='LUHa_u2.v1'):
    """

    Args:
        start_yr:
        end_yr:
        do_extend:

    Returns:

    """
    carea = util.open_or_die(constants.CELL_AREA_H)
    halfdeg = util.open_or_die(constants.input_dir + os.sep + '/public_inputs/other/miami_biomass_v3/miami_halfdeg_conform.txt')

    vba = halfdeg * 0.75  # Get above ground biomass
    vba = vba*(vba > 0.01) + (vba < 0.01)*0.01  # Set least value of vba to 0.01

    icew = util.open_or_die(constants.input_dir + os.sep + '/gicew.1700.txt', skiprows=6)

    cum_net_C_focal = []
    gross_trans_focal = []
    net_trans_focal = []
    sec_area_focal = []
    sec_age_focal = []
    wh_focal = []

    for yr in range(start_yr, end_yr):
        secd = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gsecd.' + str(yr) + '.txt', skiprows=6)
        ssmb = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gssmb.' + str(yr) + '.txt', skiprows=6)
        ssma = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gssma.' + str(yr) + '.txt', skiprows=6)
        othr = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gothr.' + str(yr) + '.txt', skiprows=6)
        crop = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gcrop.' + str(yr) + '.txt', skiprows=6)
        past = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gpast.' + str(yr) + '.txt', skiprows=6)
        flcp = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflcp.' + str(yr) + '.txt', skiprows=6)
        flpc = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflpc.' + str(yr) + '.txt', skiprows=6)
        flsp = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflsp.' + str(yr) + '.txt', skiprows=6)
        flps = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflps.' + str(yr) + '.txt', skiprows=6)
        flsc = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflsc.' + str(yr) + '.txt', skiprows=6)
        flcs = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflcs.' + str(yr) + '.txt', skiprows=6)
        flvc = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflvc.' + str(yr) + '.txt', skiprows=6)
        flvp = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gflvp.' + str(yr) + '.txt', skiprows=6)
        fvh1 = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gfvh1.' + str(yr) + '.txt', skiprows=6)
        fvh2 = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gfvh2.' + str(yr) + '.txt', skiprows=6)
        fsh1 = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gfsh1.' + str(yr) + '.txt', skiprows=6)
        fsh2 = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gfsh2.' + str(yr) + '.txt', skiprows=6)
        fsh3 = util.open_or_die(constants.input_dir + os.sep + '/LUH/' + name_LUH1 + '/updated_states/gfsh3.' + str(yr) + '.txt', skiprows=6)

        cum_net_C_focal.append(np.ma.sum(np.ma.sum(vba*(1-icew)*carea))*1e6*1e3/1e15 - np.ma.sum(np.ma.sum(secd*carea*ssmb+othr*carea*vba))*1e6*1e3/1e15)
        gross_trans_focal.append(np.ma.sum(np.ma.sum((abs(flcp)+abs(flpc)+abs(flsp)+abs(flps)+abs(flsc)+
                                           abs(flcs)+abs(flvc)+abs(flvp)+abs(fvh1)+abs(fvh2)+abs(fsh1)+abs(fsh2)+abs(fsh3))*carea)))
        net_trans_focal.append(np.ma.sum(np.ma.sum(((flsp + flsc + flvc + flvp) - (flps + flcs) + fvh1 + fvh2)*carea)))
        sec_area_focal.append(np.ma.sum(np.ma.sum(secd*carea)))
        sec_age_focal.append(np.ma.sum(np.ma.sum(secd*ssma*carea))/sum(sum((secd+1e-12)*carea)))
        wh_focal.append(np.ma.sum(abs(fvh1)+abs(fvh2)+abs(fsh1)+abs(fsh2)+abs(fsh3))*carea)

    if do_extend:
        cum_net_C_focal = add_to_list(cum_net_C_focal, start_yr, end_yr)
        gross_trans_focal = add_to_list(gross_trans_focal, start_yr, end_yr)
        net_trans_focal = add_to_list(net_trans_focal, start_yr, end_yr)
        sec_area_focal = add_to_list(sec_area_focal, start_yr, end_yr)
        sec_age_focal = add_to_list(sec_age_focal, start_yr, end_yr)
        wh_focal = add_to_list(wh_focal, start_yr, end_yr)

    return cum_net_C_focal, gross_trans_focal, net_trans_focal, sec_area_focal, sec_age_focal, wh_focal

if __name__ == '__main__':
    pass
