from __future__ import print_function, division
import os
import sys
sys.path.insert(0, os.path.abspath('../GLM/'))
import GLM.constants as constants
import GLM.process_GLM as GLM
import logging
import math
import pygeoutil.util as util
import pygeoutil.rgeo as rgeo
# Logging
cur_flname = os.path.splitext(os.path.basename(__file__))[0]
LOG_FILENAME = constants.log_dir + os.sep + 'Log_' + cur_flname + '.txt'
util.make_dir_if_missing(constants.log_dir)
logging.basicConfig(filename=LOG_FILENAME, level=logging.INFO, filemode='w',
                    format='%(asctime)s %(levelname)s %(module)s - %(funcName)s: %(message)s',
                    datefmt="%m-%d %H:%M")  # Logging levels are DEBUG, INFO, WARNING, ERROR, and CRITICAL
# Output to screen
logger = logging.getLogger(cur_flname)
if not logger.handlers:
    logger.addHandler(logging.StreamHandler(sys.stdout))

import pdb
import pandas as pd
import numpy as np


class CropFunctionalTypes:
    """

    """
    def __init__(self, res='q', skiprows=6):
        """

        :param res: Resolution of output dataset: q=quarter, h=half, o=one
        :param skiprows:
        """
        # Dictionary of crop functional types
        self.cft = {'C4Annual': ['maize.asc', 'millet.asc', 'sorghum.asc'],
                    'C4Perren': ['sugarcane.asc'],
                    'C3Perren': ['banana.asc', 'berry.asc', 'citrus.asc', 'fruittree.asc', 'grape.asc', 'palm.asc', 'tropevrgrn.asc'],
                    'Ntfixing': ['alfalfa.asc', 'bean.asc', 'legumehay.asc', 'peanut.asc', 'soybean.asc'],
                    'C3Annual': ['beet.asc', 'cassava.asc', 'cotton.asc', 'flax.asc', 'hops.asc', 'mixedcover.asc',
                                 'nursflower.asc', 'oat.asc', 'potato.asc', 'rapeseed.asc', 'rice.asc', 'rye.asc',
                                 'safflower.asc', 'sunflower.asc', 'tobacco.asc', 'vegetable.asc', 'wheat.asc'],
                    'TotlRice': ['rice.asc', 'xrice.asc']}

        # Get shape of file
        self.skiprows = skiprows
        self.res = res
        self.tmpdata = util.open_or_die(path_file=constants.DIR_MFD + os.sep + 'maize.asc', skiprows=skiprows,
                                        delimiter=' ')
        self.asc_hdr = util.get_ascii_header(path_file=constants.DIR_MFD + os.sep + 'maize.asc', getrows=skiprows)
        self.yshape = self.tmpdata.shape[0]
        self.xshape = self.tmpdata.shape[1]
        self.native_res = 180. / self.yshape

        # Create empty numpy arrays
        self.c4annual = np.zeros(shape=(self.yshape, self.xshape))
        self.c4perren = np.zeros(shape=(self.yshape, self.xshape))
        self.c3perren = np.zeros(shape=(self.yshape, self.xshape))
        self.ntfixing = np.zeros(shape=(self.yshape, self.xshape))
        self.c3annual = np.zeros(shape=(self.yshape, self.xshape))
        self.totlrice = np.zeros(shape=(self.yshape, self.xshape))
        self.totlcrop = np.zeros(shape=(self.yshape, self.xshape))

        self.c4anarea = np.zeros(shape=(self.yshape, self.xshape))
        self.c4prarea = np.zeros(shape=(self.yshape, self.xshape))
        self.c3prarea = np.zeros(shape=(self.yshape, self.xshape))
        self.ntfxarea = np.zeros(shape=(self.yshape, self.xshape))
        self.c3anarea = np.zeros(shape=(self.yshape, self.xshape))
        self.croparea = np.zeros(shape=(self.yshape, self.xshape))

        # Area of each cell in Monfreda dataset
        self.mfd_carea = rgeo.get_grid_cell_area(nrows=self.yshape, ncols=self.xshape)

        # Ice-water fraction and other static data
        # glm static data
        self.path_glm_stat = constants.path_glm_stat  # Static data, contains grid cell area (carea)
        # Get cell area (after subtracting ice/water fraction)
        nc_glm_stat = util.open_or_die(self.path_glm_stat)
        icwtr = nc_glm_stat.variables[constants.ice_water_frac][:, :]

        # Read in area file based on res
        if res == 'q':  # Quarter degree
            self.area_data = util.open_or_die(path_file=constants.CELL_AREA_Q)
            self.block_size = 0.25 / self.native_res
        elif res == 'h':  # Half degree
            self.area_data = util.open_or_die(path_file=constants.CELL_AREA_H)
            self.block_size = 0.5 / self.native_res
        elif res == 'o':  # One degree
            self.area_data = util.open_or_die(path_file=constants.CELL_AREA_O)
            self.block_size = 1.0 / self.native_res
        else:
            logger.info('Incorrect resolution for output of Monfreda')

        # Compute cell area (excluding ice-water fraction)
        self.carea = nc_glm_stat.variables['carea'][:, :]
        self.land_area = self.carea * (1.0 - icwtr)

        # Get FAO country concordance list
        self.fao_id = pd.read_csv(constants.FAO_CONCOR)[['Country_FAO', 'ISO']]

        self.CCODES = 'country code'
        self.CONT_CODES = 'continent code'

        # continent and country code files
        self.ccodes_file = constants.ccodes_file
        self.contcodes_file = constants.contcodes_file

        # Output path
        self.out_path = constants.out_dir + os.sep + 'Monfreda'
        util.make_dir_if_missing(self.out_path)

    def read_monfreda(self):
        # Loop over crop functional types
        for key, value in self.cft.items():
            for val in value:
                logger.info('Processing ' + key + ' ' + val)
                tmp_asc = util.open_or_die(path_file=constants.DIR_MFD + os.sep + val,
                                           skiprows=self.skiprows, delimiter=' ')

                if key == 'C4Annual':
                    self.c4annual = self.c4annual + tmp_asc
                elif key == 'C4Perren':
                    self.c4perren = self.c4perren + tmp_asc
                elif key == 'C3Perren':
                    self.c3perren = self.c3perren + tmp_asc
                elif key == 'Ntfixing':
                    self.ntfixing = self.ntfixing + tmp_asc
                elif key == 'C3Annual':
                    self.c3annual = self.c3annual + tmp_asc
                elif key == 'TotlRice':
                    self.totlrice = self.totlrice + tmp_asc
                else:
                    logger.info('Wrong key')

                # Add to total crop fraction of grid cell area
                self.totlcrop = self.totlcrop + tmp_asc

        # Aggregate MONFREDA data from 5' to 0.25 degree
        self.totlcrop = util.avg_np_arr(self.totlcrop, block_size=3)
        self.croparea = self.totlcrop * self.land_area

        # Aggregate MONFREDA data for each CFT from 5' to 0.25 degree
        self.c4anarea = util.avg_np_arr(self.c4annual, area_cells=self.mfd_carea, block_size=3)
        self.c4prarea = util.avg_np_arr(self.c4perren, area_cells=self.mfd_carea, block_size=3)
        self.c3prarea = util.avg_np_arr(self.c3perren, area_cells=self.mfd_carea, block_size=3)
        self.ntfxarea = util.avg_np_arr(self.ntfixing, area_cells=self.mfd_carea, block_size=3)
        self.c3anarea = util.avg_np_arr(self.c3annual, area_cells=self.mfd_carea, block_size=3)

    def read_HYDE(self):
        pass

    def output_ascii_to_file(self, fl_name, data, delim=' '):
        asc_file = open(fl_name, 'w+')

        if self.res == 'q':
            ncols = 720
            nrows = 360
            cell_size = 0.25
        elif self.res == 'h':
            # @TODO
            pass
        elif self.res == 'o':
            # @TODO
            pass

        asc_file.write('ncols         %s\n' % ncols)
        asc_file.write('nrows         %s\n' % nrows)
        asc_file.write('xllcorner     -180\n')
        asc_file.write('yllcorner     -90\n')
        asc_file.write('cellsize         %s\n' % cell_size)
        asc_file.write('NODATA_value  -9999\n')

        # Write numpy array
        np.savetxt(asc_file, data, delimiter=delim)
        asc_file.close()

    def compute_area_by_country(self):
        """
        Output area of each CFT by country
        :return:
        """
        df = []
        # Read ASCII file of country codes
        ccodes_fl = util.open_or_die(constants.CNTRY_CODES, skiprows=0, delimiter=' ')

        # Get list of unique countries, remove 0.0 as it is not a country ID
        list_cntrs = np.unique(ccodes_fl)
        list_cntrs = list_cntrs[list_cntrs > 0.0]

        # For each country:
        for cnt in list_cntrs:
            # Get area of cropland in country based on MONFREDA data
            area_cnt = self.croparea[ccodes_fl[:] == cnt].sum()
            area_c4a = self.c4anarea[ccodes_fl[:] == cnt].sum()
            area_c4p = self.c4prarea[ccodes_fl[:] == cnt].sum()
            area_c3p = self.c3prarea[ccodes_fl[:] == cnt].sum()
            area_ntf = self.ntfxarea[ccodes_fl[:] == cnt].sum()
            area_c3a = self.c3anarea[ccodes_fl[:] == cnt].sum()

            # Get country name from concordance table
            cnt_name = self.fao_id[self.fao_id['ISO'] == int(cnt)]['Country_FAO'].iloc[0]
            df.append({'ISO': int(cnt),
                       'Country_Monfreda': cnt_name,
                       'Area_Monfreda': area_cnt,
                       'Monfreda_c4annual': area_c4a,
                       'Monfreda_c4perren': area_c4p,
                       'Monfreda_c3perren': area_c3p,
                       'Monfreda_ntfixing': area_ntf,
                       'Monfreda_c3annual': area_c3a})

        return pd.DataFrame(df)


    def create_rotations_nc(self, df):
        """
        :param df: Pandas dataframe
            Dataframe containing cropland area for each country x functional crop type combination
            Country_FAO    functional crop type     mean_area
            Albania             C3annual         6.687115e+03
            Albania          C3perennial         4.139867e+03
            Albania             C4annual         5.300000e+04
            Albania             N-fixing         4.460714e+03
            Algeria             C3annual         5.371344e+04
        :return:
        """
        logger.info('create_rotations_nc')

        # Read in country and continent file
        # Create dataframe combining country and continent files
        ccodes = pd.read_csv(self.ccodes_file, header=None)
        ccodes.columns = [self.CCODES]
        contcodes = pd.read_csv(self.contcodes_file, header=None)
        contcodes.columns = [self.CONT_CODES]
        lup_codes = pd.concat([ccodes, contcodes], axis=1)


        out_nc = constants.path_glm_output + os.sep + 'national_crop_rotation_data_850_2015_new_sept.nc'
        nc_data = util.open_or_die(out_nc, perm='w', format='NETCDF4_CLASSIC')
        nc_data.description = ''

        # dimensions
        tme = np.arange(constants.GLM_STRT_YR, constants.GLM_END_YR + 1)

        nc_data.createDimension('time', np.shape(tme)[0])
        nc_data.createDimension('country', len(ccodes))

        # Populate and output nc file
        time = nc_data.createVariable('time', 'i4', ('time',), fill_value=0.0)
        country = nc_data.createVariable('country', 'i4', ('country',), fill_value=0.0)

        # Assign units and other metadata
        time.units = 'year as %Y.%f'
        time.calendar = 'proleptic_gregorian'
        country.units = 'ISO country code'

        # Assign values to dimensions and data
        time[:] = tme
        country[:] = ccodes.values

        c4ann_to_c3nfx = nc_data.createVariable('c4ann_to_c3nfx', 'f4', ('time', 'country',))
        c4ann_to_c3ann = nc_data.createVariable('c4ann_to_c3ann', 'f4', ('time', 'country',))
        c3ann_to_c3nfx = nc_data.createVariable('c3ann_to_c3nfx', 'f4', ('time', 'country',))

        c4ann_to_c3nfx.units = 'fraction of crop type area undergoing crop rotation'
        c4ann_to_c3nfx.long_name = 'Crop rotations: C4 Annual, C3 N-Fixing'

        c4ann_to_c3ann.units = 'fraction of crop type area undergoing crop rotation'
        c4ann_to_c3ann.long_name = 'Crop rotations: C4 Annual, C3 Annual'

        c3ann_to_c3nfx.units = 'fraction of crop type area undergoing crop rotation'
        c3ann_to_c3nfx.long_name = 'Crop rotations: C3 Annual, C3 N-Fixing'

        # Loop over all countries
        for index, row in lup_codes.iterrows():
            # print index, row[self.CCODES], row[self.CONT_CODES]

            # Find row containing country in df_merge
            row_country = df[df['id'] == row[self.CCODES]]
            if len(row_country):
                c4ann_to_c3nfx[:, index] = row_country['c4ann_to_c3nfx'].values[0]
                c4ann_to_c3ann[:, index] = row_country['c4ann_to_c3ann'].values[0]
                c3ann_to_c3nfx[:, index] = row_country['c3ann_to_c3nfx'].values[0]
            else:
                # TODO Find the average crop rotation rate for the continent
                c4ann_to_c3nfx[:, index] = 0.03  # 0.53
                c4ann_to_c3ann[:, index] = 0.01
                c3ann_to_c3nfx[:, index] = 0.02

        nc_data.close()


if __name__ == '__main__':
    import datetime
    obj = CropFunctionalTypes(res='q', skiprows=6)

    #obj.read_monfreda()
    #pdb.set_trace()
    c3annual = util.open_or_die(obj.out_path + os.sep + 'c3annual.txt')
    c4annual = util.open_or_die(obj.out_path + os.sep + 'c4annual.txt')
    ntfixing = util.open_or_die(obj.out_path + os.sep + 'ntfixing.txt')

    mat_rotations = np.stack((c3annual, c4annual, ntfixing))  # shape: (3L, 2160L, 4320L)

    c4ann_to_ntfix = np.zeros_like(c3annual)
    c4ann_to_c3ann = np.zeros_like(c3annual)
    c3ann_to_ntfix = np.zeros_like(c3annual)

    # Compute crop rotations
    # 0: c3annual, 1:c4annual, 2:ntfixing
    idx = 0
    for y, z in np.ndindex(mat_rotations.shape[1:]):
        idx += 1
        if idx % 1000000 == 0.0:
            print(idx, datetime.datetime.now().time())

        rot = mat_rotations[:, y, z].data
        # Case 0. All values are 0
        # Case 1. c3annual <= c4annual <= ntfixing
        # Case 2. c3annual <= ntfixing <= c4annual
        # Case 3. c4annual <= c3annual <= ntfixing
        # Case 4. c4annual <= ntfixing <= c3annual
        # Case 5. ntfixing <= c3annual <= c4annual
        # Case 6. ntfixing <= c4annual <= c3annual
        if np.all(rot == 0.0):
            pass
        elif (rot[0] <= rot[1] <= rot[2]):
            # c3annual <= c4annual <= ntfixing
            denom = (rot[1] + rot[2])
            c4ann_to_c3ann[y, z] = rot[0] * rot[1] / denom  # Fraction of c3ann going to c4ann
            c3ann_to_ntfix[y, z] = rot[0] * rot[2] / denom  # Fraction of c3ann going to ntfix
            c4ann_to_ntfix[y, z] = (rot[1] - c4ann_to_c3ann[y, z])  # Fraction of c4ann going to ntfix
        elif (rot[0] <= rot[2] <= rot[1]):
            # c3annual <= ntfixing <= c4annual
            denom = (rot[1] + rot[2])
            c3ann_to_ntfix[y, z] = rot[0] * rot[2] / denom  # Fraction of c3ann going to ntfix
            c4ann_to_c3ann[y, z] = rot[0] * rot[1] / denom  # Fraction of c3ann going to c4ann
            c4ann_to_ntfix[y, z] = (rot[2] - c3ann_to_ntfix[y, z])  # Fraction of c4ann going to ntfix
        elif (rot[1] <= rot[0] <= rot[2]):
            # c4annual <= c3annual <= ntfixing
            denom = (rot[0] + rot[2])
            c4ann_to_c3ann[y, z] = rot[1] * rot[0] / denom  # Fraction of c4ann going to c3ann
            c4ann_to_ntfix[y, z] = rot[1] * rot[2] / denom  # Fraction of c4ann going to ntfix
            c3ann_to_ntfix[y, z] = (rot[0] - c4ann_to_c3ann[y, z])  # Fraction of c4ann going to ntfix
        elif (rot[1] <= rot[2] <= rot[0]):
            # c4annual <= ntfixing <= c3annual
            denom = (rot[0] + rot[2])
            c4ann_to_ntfix[y, z] = rot[1] * rot[2] / denom  # Fraction of c4ann going to ntfix
            c4ann_to_c3ann[y, z] = rot[1] * rot[0] / denom  # Fraction of c4ann going to c3ann
            c3ann_to_ntfix[y, z] = (rot[2] - c4ann_to_ntfix[y, z])  # Fraction of c4ann going to ntfix
        elif (rot[2] <= rot[0] <= rot[1]):
            # ntfixing <= c3annual <= c4annual
            denom = (rot[0] + rot[1])
            c3ann_to_ntfix[y, z] = rot[2] * rot[0] / denom  # Fraction of ntfix going to c3ann
            c4ann_to_ntfix[y, z] = rot[2] * rot[1] / denom  # Fraction of ntfix going to c4ann
            c4ann_to_c3ann[y, z] = (rot[0] - c3ann_to_ntfix[y, z])  # Fraction of c4ann going to c3ann
        else:
            # (rot[2] <= rot[1] <= rot[0]):
            # ntfixing <= c4annual <= c3annual
            denom = (rot[0] + rot[1])
            c4ann_to_ntfix[y, z] = rot[2] * rot[1] / denom  # Fraction of ntfix going to c4ann
            c3ann_to_ntfix[y, z] = rot[2] * rot[0] / denom  # Fraction of ntfix going to c3ann
            c4ann_to_c3ann[y, z] = (rot[1] - c4ann_to_ntfix[y, z])  # Fraction of c4ann going to c3ann

    print((c4ann_to_ntfix.data > 1.0).sum())
    print((c3ann_to_ntfix.data > 1.0).sum())
    print((c4ann_to_c3ann.data > 1.0).sum())

    # Clip arrays between 0.0 and 1.0
    c4ann_to_ntfix = np.clip(c4ann_to_ntfix, 0.0, 1.0)
    c3ann_to_ntfix = np.clip(c3ann_to_ntfix, 0.0, 1.0)
    c4ann_to_c3ann = np.clip(c4ann_to_c3ann, 0.0, 1.0)
    #pdb.set_trace()
    # Convert from 5' (native resolution) to 0.25 degree (new resolution)
    #new_res_area = util.avg_np_arr(obj.mfd_carea, block_size=int(obj.block_size), func=np.ma.sum)
    c4ann_to_ntfix = util.avg_np_arr(c4ann_to_ntfix, block_size=int(obj.block_size))
    c3ann_to_ntfix = util.avg_np_arr(c3ann_to_ntfix, block_size=int(obj.block_size))
    c4ann_to_c3ann = util.avg_np_arr(c4ann_to_c3ann, block_size=int(obj.block_size))

    # Divide by area
    #c4ann_to_ntfix = c4ann_to_ntfix / new_res_area
    #c3ann_to_ntfix = c3ann_to_ntfix / new_res_area
    #c4ann_to_c3ann = c4ann_to_c3ann / new_res_area

    exp = constants.glm_experiments[0]
    obj_g = GLM.glm(name_exp=exp,
                    path_input=constants.path_glm_input + os.sep + exp,
                    time_period='past',
                    out_path=constants.path_glm_output + os.sep + exp + os.sep + 'states')
    pdb.set_trace()
    df_a = obj_g.get_arr_by_region(c4ann_to_ntfix, obj.land_area, do_avg=True, do_area=False,
                                   name_area_col='c4ann_to_c3nfx', type_region='country')
    df_b = obj_g.get_arr_by_region(c3ann_to_ntfix, obj.land_area, do_avg=True, do_area=False,
                                   name_area_col='c3ann_to_c3nfx', type_region='country')
    df_c = obj_g.get_arr_by_region(c4ann_to_c3ann, obj.land_area, do_avg=True, do_area=False,
                                   name_area_col='c4ann_to_c3ann', type_region='country')

    df = pd.concat([df_a, df_b], axis=1, join='inner')
    dups = util.duplicate_columns(df)
    df = df.drop(dups, axis=1)

    df = pd.concat([df, df_c], axis=1, join='inner')  # Add dataframe c
    pdb.set_trace()
    obj.create_rotations_nc(df)
    pdb.set_trace()
    mat_rotations = np.stack((c3annual, c4annual, ntfixing))  # shape: (3L, 2160L, 4320L)
    mat_rotations = mat_rotations.T.reshape([mat_rotations.shape[1] * mat_rotations.shape[2], 3])


    ####### (y) 4320 ############
    #
    # (x)
    # 2
    # 1
    # 6
    # 0
    #
    #
    ####### 4320 ############
    idx = 0
    for x in range(2160):
        for y in range(4320):
            idx += 1
            if idx % 1000 == 0.0:
                print(idx)
            # print(x, y, mat_rotations[:, x, y])
    # get_arr_by_region
    pdb.set_trace()
    for (z, x, y), value in np.ndenumerate(mat_rotations):
        # Find
        idx += 1
        if idx == 4340:
            pdb.set_trace()
            # c3ann_to_c4ann[x, y] = 10.0
        print(z, x, y, value)

    pdb.set_trace()
    for y, z in np.ndindex(mat_rotations.shape[1:]):
        print(y, z, mat_rotations[:, y, z])

    np.savetxt(obj.out_path + os.sep + 'c3annual.txt', obj.c3annual, fmt='%.6f', delimiter=' ')
    np.savetxt(obj.out_path + os.sep + 'c4annual.txt', obj.c4annual, fmt='%.6f', delimiter=' ')
    np.savetxt(obj.out_path + os.sep + 'ntfixing.txt', obj.ntfixing, fmt='%.6f', delimiter=' ')
    pdb.set_trace()
    obj.read_monfreda()
    dict_rotations = {'c3annual': 0, 'c4annual': 1, 'ntfixing': 2}

    # Create a rotation matrix by substituting space for time
    mat_rotations = np.stack((obj.c3annual, obj.c4annual, obj.ntfixing))

    # Find the crop occuping the minimum area
    minima = np.argmin(mat_rotations, axis=0)  # contains a 5x5 array of 0,1,2 for a,b,c respectively

    # 1. c3ann >= c4ann >= ntfix
    ntfix_to_c4ann = obj.ntfixing * obj.c4annual / (obj.c3annual + obj.c4annual)
    ntfix_to_c4ann[minima != dict_rotations['ntfixing']] = 0
    ntfix_to_c4ann = obj.ntfixing - ntfix_to_c4ann

    # 2. c3ann >= ntfix >= c4ann
    # 3. c4ann >= c3ann >= ntfix
    # 4. c4ann >= ntfix >= c3ann
    # 5. ntfix >= c4ann >= c3ann
    # 6. ntfix >= c3ann >= c4ann

    # For cells where ntfix occupies least area, find rotations
    ntfix_to_c4ann = obj.ntfixing * obj.c4annual / (obj.c3annual + obj.c4annual)
    ntfix_to_c4ann[minima != dict_rotations['ntfixing']] = 0
    ntfix_to_c3ann = obj.ntfixing - ntfix_to_c4ann

    # Compute the remaining rotation
    c4ann_to_c3ann = min(obj.c3annual - ntfix_to_c3ann, obj.c4annual - ntfix_to_c4ann)

    # For cells where c4ann occupies least area, find rotations
    c4ann_to_c3ann = obj.c4annual * obj.c3annual / (obj.ntfixing + obj.c4annual)
    c3ann_to_ntfix = obj.c4annual - c4ann_to_c3ann
    ntfix_to_c4ann = min(obj.c3annual - c3ann_to_ntfix, obj.c4annual - c4ann_to_c3ann)

    # TODO For cells where c3ann occupies least area, find rotations
    c3ann_to_c4ann = obj.c3annual * obj.c4annual / (obj.ntfixing + obj.c3annual)
    c3ann_to_ntfix = obj.c4annual - c4ann_to_c3ann
    ntfix_to_c4ann = min(obj.c3annual - c3ann_to_ntfix, obj.c4annual - c4ann_to_c3ann)

    # Subtract from remaining
    pdb.set_trace()
    df = obj.compute_area_by_country()
    df.to_csv(obj.out_path + os.sep + 'Monfreda_area_crops.csv')

    # Output crop area to file
    obj.output_ascii_to_file(fl_name=obj.out_path + os.sep + 'MONFREDA_crop_area.asc', data=obj.croparea, delim=' ')

    # Read in FAO data for entire time period
    fao_obj = crop_stats.CropStats()
    fao_obj.process_crop_stats()

    # Compare FAO and Monfreda
    # fao_df.FAO_perc_all_df

    # Convert from ascii files to our crop functional types

    # Aggregate ascii files to quarter degree

    # Produce netCDF file as output
    # Currently, we compute crop type ratio's for the year 2000 based on Monfreda data crop type ratios and ag area
    # from HYDE. This gives us a static number per CFT per grid cell. However, sometimes, the crop type ratio might be 0
    # because Monfreda says that no crop exists, but HYDE says that crops do exist.
    # New approach will:
    # 1. Compute CFT fraction per grid cell in 2000
    # 2. Aggregate this fractional value by country
    # 3. Apply this fractional value for grid cells in the past where Monfreda says that no data exists. The values are
    #    assigned by country

    # 1. Read in country codes file
    # 2. Determine list of unique countries
    # 3. For each country:
    # 3a.   Compute fraction of each CFT for that particular country in 2000 by averaging
    # 4. Use the HYDE gcrop dataset so that for each grid cell where a crop is present, we provide either:
    #    a. Fraction as determined through Monfreda
    #    b. Country-specific fraction for grid cells where Monfreda says that there is no cropping
    # 5. Output is a netCDF file with dimensions: time x lat x lon which provides CFT fraction for 5 CFTs across time
    #    and for each grid cell

    # 1) We need to replace that 1/5th for each crop type with something more reasonable from the FAO data. So, we need
    # to get FAO national ratios for each crop type for years 1961-2014 and then perhaps hold constant for 2015 and use
    # 1961-1965 averages for years prior to 1961. This will give us a 199 x 5 x 516 file (could be netcdf or other.

    # 2) The next step is a bit more complicated. In this step we need to use the FAO data to modify the Monfreda maps
    # for years before/after 2000. I think the first step here is to compare Monfreda and FAO national crop type ratios
    # in year 2000. We can then 'normalize' the FAO values so that in the year 2000 FAO national ratios are the same as
    # Monfreda. Using these normalized values we can modify the Monfreda map forwards and backwards in time (throughout
    # the FAO period) by changing all grid-cells within a country using the new FAO ratios. Prior to 1961 we would hold
    # the map constant. The input to GLM would be a lat x lon x time x 5 file.

    # This second step is definitely more complicated so we just need to break it down into manageable chunks. I would
    # not worry about forming a data cube at this point. We first need to be looking at the data and seeing how we can
    # connect the FAO and Monfreda data together.